package com.jz.dm.filter;

import com.alibaba.fastjson.JSONObject;
import com.jz.common.utils.RedisUtils;
import com.jz.dm.common.constant.Constants;
import com.jz.dm.common.constant.LoggingConstants;
import com.jz.dm.common.enums.GatewayResultCode;
import com.jz.dm.common.enums.apiInterface.ApiStatusEnum;
import com.jz.dm.common.enums.org.OrgStatusEnum;
import com.jz.dm.common.exception.GatewayException;
import com.jz.dm.common.util.DateUtil;
import com.jz.dm.models.domian.ApiAuth;
import com.jz.dm.models.domian.ApiInterface;
import com.jz.dm.models.domian.ApiOrg;
import com.jz.dm.models.enity.GatewayRequest;
import com.jz.dm.models.enity.GatewayResponse;
import com.jz.dm.service.ApiInterfaceService;
import com.jz.dm.service.AuthService;
import com.jz.dm.service.OrganizationManageService;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.stereotype.Component;

import java.text.ParseException;
import java.text.SimpleDateFormat;
import java.util.Date;

/**
 * @author ZC
 * @PACKAGE_NAME: com.jz.dm.filter
 * @PROJECT_NAME: jz-dm-parent
 * @NAME: AuthFilter
 * @DATE: 2020-12-23/17:43
 * @DAY_NAME_SHORT: 周三
 * @Description: 认证过滤器
 **/
@Component("authFilter")
@Slf4j
public class AuthFilter extends AbstractFilter {

    @Autowired
    private AuthService authService;
    @Autowired
    private OrganizationManageService organizationManageService;
    @Autowired
    private ApiInterfaceService apiInterfaceService;
    @Autowired
    private RedisUtils redisUtils;

    @Value("${request.limit.max}")
    private Long limit;

    @Override
    public String getFilterName() {
        return "AuthFilter";
    }

    @Override
    public int getOrder() {
        return Constants.FILTER_ORDER_5;
    }

    @Override
    protected void internalDoFilter(GatewayRequest request, GatewayResponse response,
                                    FilterChain chain) {
        try {
            ApiInterface apiInterface = apiInterfaceService.getApiInfo(request.getApiKey());
            if (null == apiInterface) {
                throw new GatewayException(GatewayResultCode.ILLEGAL_REQUEST);
            }
            if (!ApiStatusEnum.ISSUE.name().equals(apiInterface.getStatus())) {
                throw new GatewayException(GatewayResultCode.API_STATUS_EXCEPTION);
            }
            JSONObject parameter = JSONObject.parseObject(request.getParams());
            if (null != parameter) {
                String authCode = parameter.getString("authCode");
                ApiAuth authAuth = authService.getAuthUser(authCode,apiInterface.getId());
                if (null == authAuth) {
                    throw new GatewayException(GatewayResultCode.REQUEST_NOT_AUTH);
                }
                //查询认证组织信息
                ApiOrg apiOrg = organizationManageService.getAuthOrganization(authAuth.getApiOrgId());
                if (null == apiOrg) {
                    throw new GatewayException(GatewayResultCode.ORG_NOT_EXIST);
                }
                checkBill(request, authCode, authAuth, apiOrg);
            }
        } catch (Exception e) {
            log.error("~~~~~~~~~~~~~~~~认证过滤异常~~~~~~~~~~~~~~~~~");
            log.error("异常信息:{}", e.getMessage());
        }
        chain.doFilter(request, response);
    }

    /**
     * 校验计费信息
     * @param request
     * @param authCode
     * @param authAuth
     * @param apiOrg
     * @throws ParseException
     */
    private void checkBill(GatewayRequest request, String authCode,
                           ApiAuth authAuth, ApiOrg apiOrg) throws ParseException {
        switch (authAuth.getAuthMode()) {
            case "POWER_CALL_MODE": //按次调用
                //状态检查(只限制按次请求$$ 不限制按时间请求)
                if (!OrgStatusEnum.NORMAL.name().equals(apiOrg.getStatus())) {
                    throw new GatewayException(GatewayResultCode.ORG_STATE_EXCEPTION);
                }
                //查询数据银行银行余额是否充足
                try { //记录请求次数(每天限制请求次数)
                    String limitKey = request.getApiKey() + LoggingConstants.AND_SPILT + authCode;
                    String reqKey = redisUtils.get(limitKey);
                    long timeOut = DateUtil.calculateNowResidueTime();
                    if (null != reqKey) {
                        Integer value = Integer.valueOf(reqKey);
                        if (value > limit) {//超出最大请求次数
                            throw new GatewayException(GatewayResultCode.REQUEST_LIMIT_EXCEPTION);
                        } else if (value <= limit) {
                            redisUtils.delAndAdd(limitKey, limitKey, value + 1, timeOut);
                        }
                    } else {
                        redisUtils.set(limitKey, 1, timeOut);
                    }
                } catch (Exception ex) {
                    log.error("~~~~~~~~~~~~~~~~~~~~~~过滤计次信息异常~~~~~~~~~~~~~~~~~~~");
                    log.error("异常信息:{}", ex.getMessage());
                }
                break;
            case "RECORD_TIME_MODE": //按时间调用
                SimpleDateFormat dateFormat = new SimpleDateFormat(Constants.DATE_TIME_FORMAT);
                String formatDate = dateFormat.format(new Date());
                Date currentDate = dateFormat.parse(formatDate);
                Date validEndTime = authAuth.getValidEndTime();
                //如果 date1 在 date2 之前，before 返回 true，否则返回 false
                if (currentDate.before(validEndTime)) {//超出时间
                    throw new GatewayException(GatewayResultCode.RATE_LIMIT_EXCEEDED);
                }
                break;
            case "PERMANENT_TIME_MODE"://永久有效(直接跳出)
                break;
            default:
                throw new GatewayException(GatewayResultCode.ILLEGAL_REQUEST);
        }
    }
}
