/**
 * Copyright (c) 2011-2019 All Rights Reserved.
 */
package com.jz.common.utils;

import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.data.redis.core.RedisTemplate;
import org.springframework.data.redis.core.script.RedisScript;
import org.springframework.stereotype.Component;

import java.util.List;
import java.util.Map;
import java.util.Set;
import java.util.concurrent.TimeUnit;

/**
 *
 *
 * @author Admin
 * @version $Id: RedisUtils.java 2019年11月2日 上午8:36:41 $
 */
@Component("redisUtils")
@Slf4j
public class RedisUtils {
	@Autowired
	private RedisTemplate<String, Object> redisTemplate;

	/**
	 * 实现命令：TTL key，以秒为单位，返回给定 key的剩余生存时间(TTL, time to live)。
	 *
	 * @param key
	 * @return
	 */
	public long ttl(String key) {
		return redisTemplate.getExpire(key);
	}

	/**
	 * 实现命令：expire 设置过期时间，单位秒
	 *
	 * @param key
	 * @return
	 */
	public void expire(String key, long timeout) {
		redisTemplate.expire(key, timeout, TimeUnit.SECONDS);
	}

	/**
	 * 实现命令：INCR key，增加key一次
	 *
	 * @param key
	 * @return
	 */
	public long incr(String key, long delta) {
		return redisTemplate.opsForValue().increment(key, delta);
	}

	/**
	 * 实现命令： key，减少key一次
	 *
	 * @param key
	 * @return
	 */
	public long decr(String key, long delta) {
		if (delta < 0) {
			del(key);
			return 0;
		}
		return redisTemplate.opsForValue().increment(key, -delta);
	}
	
	/**
	 * 实现命令：KEYS pattern，查找所有符合给定模式 pattern的 key
	 */
	public Set<String> keys(String pattern) {
		return redisTemplate.keys(pattern);
	}

	/**
	 * 实现命令：DEL key，删除一个key
	 *
	 * @param key
	 */
	public void del(String key) {
		redisTemplate.delete(key);
	}

	/**
	 * 实现命令：SET key value，设置一个key-value（将字符串值 value关联到 key）
	 *
	 * @param key
	 * @param value
	 */
	public void set(String key, Object value) {
		redisTemplate.opsForValue().set(key, value);
	}

	/**
	 * 实现命令：SET key value EX seconds，设置key-value和超时时间（秒）
	 *
	 * @param key
	 * @param value
	 * @param timeout （以秒为单位）
	 */
	public void set(String key, Object value, long timeout) {
		redisTemplate.opsForValue().set(key, value, timeout, TimeUnit.SECONDS);
	}
	/**
	 * 实现命令：SET key value EX seconds，设置key-value和超时时间（分钟）
	 *
	 * @param key
	 * @param value
	 * @param timeout （以分钟为单位）
	 */
	public Boolean setIfAbsent(String key, String value, long timeout) {
		try {
			redisTemplate.opsForValue().set(key, value, timeout, TimeUnit.MINUTES);
		} catch(Exception e) {
			e.printStackTrace();
			return false;
		}
		return true;
	}
	/**
	 * 实现命令：GET key，返回 key所关联的字符串值。
	 *
	 * @param key
	 * @return value
	 */
	public String get(String key) {
		return (String) redisTemplate.opsForValue().get(key);
	}

	/**
	 * Hash（哈希表）
	 * 实现命令：HSET key field value，将哈希表 key中的域 field的值设为 value
	 *
	 * @param key
	 * @param field
	 * @param value
	 */
	public void hset(String key, String field, Object value) {
		redisTemplate.opsForHash().put(key, field, value);
	}
	
	/**
	 * Hash（哈希表）
	 * 实现命令：HMSET key map 批量插入
	 * 
	 * @param key
	 * @param m
	 */
	public void hmset(String key, Map<Object, Object> m) {
		redisTemplate.opsForHash().putAll(key, m);
	}

	/**
	 * 实现命令：HGET key field，返回哈希表 key中给定域 field的值
	 *
	 * @param key
	 * @param field
	 * @return
	 */
	public String hget(String key, String field) {
		return (String) redisTemplate.opsForHash().get(key, field);
	}

	/**
	 * 实现命令：HDEL key field [field ...]，删除哈希表 key 中的一个或多个指定域，不存在的域将被忽略。
	 *
	 * @param key
	 * @param fields
	 */
	public void hdel(String key, Object... fields) {
		redisTemplate.opsForHash().delete(key, fields);
	}

	/**
	 * 实现命令：HGETALL key，返回哈希表 key中，所有的域和值。
	 *
	 * @param key
	 * @return
	 */
	public Map<Object, Object> hgetall(String key) {
		return redisTemplate.opsForHash().entries(key);
	}

	/**
	 * List（列表）
	 * 实现命令：LPUSH key value，将一个值 value插入到列表 key的表头
	 *
	 * @param key
	 * @param value
	 * @return 执行 LPUSH命令后，列表的长度。
	 */
	public long lpush(String key, Object value) {
		return redisTemplate.opsForList().leftPush(key, value);
	}

	/**
	 * 实现命令：LPOP key，移除并返回列表 key的头元素。
	 *
	 * @param key
	 * @return 列表key的头元素。
	 */
	public String lpop(String key) {
		return (String) redisTemplate.opsForList().leftPop(key);
	}

	/**
	 * 实现命令：RPUSH key value，将一个值 value插入到列表 key的表尾(最右边)。
	 *
	 * @param key
	 * @param value
	 * @return 执行 LPUSH命令后，列表的长度。
	 */
	public long rpush(String key, Object value) {
		return redisTemplate.opsForList().rightPush(key, value);
	}
	
	/**
	 * 
	 * 
	 * @param <T>
	 * @param script
	 * @param keys
	 * @param args
	 * @return
	 */
	public <T> T execute(RedisScript<T> script, List<String> keys, Object... args) {
		return redisTemplate.execute(script, keys, args);
	}

	/**
	 * 
	 * @param
	 */
	public List<Object> range(String key) {
		return redisTemplate.opsForList().range(key, 0, 500);
	}

	/**
	 * 删除久key值,添加新key值
	 * @param nowKey
	 * @param newKey
	 * @param value
	 * @param timeout
	 * @return
	 */
	public void delAndAdd(String nowKey,String newKey,Integer value,long timeout){
		try {
			redisTemplate.delete(nowKey);
			redisTemplate.opsForValue().set(newKey,value,timeout);
		}catch (Exception e) {
			log.error("~~~~~~~~~~~~~保存redis计次异常~~~~~~~~~~~~~~~~~~~");
		    e.printStackTrace();
		}
	}
}
